# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
from greenbyteapi.configuration import Configuration
from greenbyteapi.controllers.base_controller import BaseController
from greenbyteapi.http.auth.custom_header_auth import CustomHeaderAuth
from greenbyteapi.models.device import Device
from greenbyteapi.models.site_with_data import SiteWithData
from greenbyteapi.models.power_curve import PowerCurve
from greenbyteapi.exceptions.problem_details_exception import ProblemDetailsException
from greenbyteapi.exceptions.api_exception import APIException

class AssetsController(BaseController):

    """A Controller to access Endpoints in the greenbyteapi API."""


    def get_devices(self,
                    device_type_ids=None,
                    site_ids=None,
                    parent_ids=None,
                    fields=None,
                    page_size=50,
                    page=1,
                    use_utc=False):
        """Does a GET request to /devices.

        Gets a list of devices that the API key has permissions for.
        _🔐 This endpoint requires the **Assets** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `devices.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_type_ids (list of int, optional): Only include devices of
                these types. Examples: * 1 - Wind turbine * 2 - Production
                meter * 3 - Met mast * 4 - Inverter * 10 - Device group * 11 -
                Grid meter * 12 - Combiner box * 23 - String * 27 - Virtual
                Meteo Sensor
            site_ids (list of int, optional): Only include devices at these
                sites.
            parent_ids (list of int, optional): Only include devices with
                these parent devices.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `Device`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of Device: Response from the API. A list of devices with
                associated metadata.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/devices'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceTypeIds': device_type_ids,
            'siteIds': site_ids,
            'parentIds': parent_ids,
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, Device.from_dictionary)

    def get_sites(self,
                  fields=None,
                  page_size=50,
                  page=1):
        """Does a GET request to /sites.

        Gets a list of sites that the API key has permissions
        for.
        _🔐 This endpoint requires the **Assets** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `sites.json` and 
        a JSON request body instead of query parameters._

        Args:
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `SiteWithData`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.

        Returns:
            list of SiteWithData: Response from the API. A list of sites with
                associated metadata.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/sites'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'fields': fields,
            'pageSize': page_size,
            'page': page
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, SiteWithData.from_dictionary)

    def get_power_curves(self,
                         device_ids,
                         timestamp=None,
                         learned=False):
        """Does a GET request to /powercurves.

        Gets the default or learned power curves for wind turbines.
        Other device types are not supported.
        _🔐 This endpoint requires the **PowerCurves** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `powercurves.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): What devices to get power curves for.
                Only wind turbines are supported.
            timestamp (date, optional): The date for which to get power
                curves. The default is the current date.
            learned (bool, optional): Whether to get learned power curves
                instead of default power curves.

        Returns:
            list of PowerCurve: Response from the API. A list of objects
                containing device ids and associated power curves.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/powercurves'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids,
            'timestamp': timestamp,
            'learned': learned
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, PowerCurve.from_dictionary)



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class TaskStateEnum(object):

    """Implementation of the 'TaskState' enum.

    The state of a task.

    Attributes:
        UNRESOLVED: TODO: type description here.
        RESOLVED: TODO: type description here.

    """

    UNRESOLVED = 'unresolved'

    RESOLVED = 'resolved'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.models.user
import greenbyteapi.models.task_category
import greenbyteapi.models.task_comment
import greenbyteapi.models.recurrence
import greenbyteapi.models.metadata_field

class Task(object):

    """Implementation of the 'Task' model.

    A task.

    Attributes:
        task_id (int): The id of a task.
        title (string): TODO: type description here.
        created_by (User): TODO: type description here.
        description (string): TODO: type description here.
        category (TaskCategory): Basic information about a task category.
        priority (TaskPriorityEnum): The priority of a task.
        timestamp_start (datetime): The timestamp when the task is/was planned
            to start. The timestamp is in the time zone configured in the
            Greenbyte Platform without UTC offset.
        timestamp_end (datetime): The timestamp when the is/was planned to
            end. The timestamp is in the time zone configured in the Greenbyte
            Platform without UTC offset.
        state (TaskStateEnum): The state of a task.
        resolved (bool): TODO: type description here.
        timestamp_resolved (datetime): The timestamp when the task was
            resolved. The timestamp is in the time zone configured in the
            Greenbyte Platform without UTC offset.
        device_ids (list of int): Ids of the devices assigned to the task.
        site_ids (list of int): Ids of the sites assigned to the task.
        site_access_ids (list of int): Ids of the site accesses linked to the
            task.
        downtime_event_ids (list of int): Ids of the downtime events linked to
            the task.
        status_ids (list of int): Ids of the statuses linked to the task.
        number_of_comments (int): TODO: type description here.
        comments (list of TaskComment): The comments belonging to the task.
        recurrence (Recurrence): Recurrence settings for the task. To
            calculate when the task is recurring, use the `timestampStart`
            field and then add to it multiples of the specified interval; the
            `intervalType` field determines if the task is recurring on daily,
            weekly, monthly, or yearly basis.  If the task is not recurring,
            this field is null.  **Note:** Only the main (first) task in a
            recurring series have recurrence settings. For the other tasks in
            the series, the field `mainTaskId` can be used to find it.
        main_task_id (int): TODO: type description here.
        assignee (object): TODO: type description here.
        metadata (list of MetadataField): A list of metadata fields and their
            values.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "task_id":'taskId',
        "title":'title',
        "created_by":'createdBy',
        "priority":'priority',
        "timestamp_start":'timestampStart',
        "timestamp_end":'timestampEnd',
        "state":'state',
        "resolved":'resolved',
        "number_of_comments":'numberOfComments',
        "description":'description',
        "category":'category',
        "timestamp_resolved":'timestampResolved',
        "device_ids":'deviceIds',
        "site_ids":'siteIds',
        "site_access_ids":'siteAccessIds',
        "downtime_event_ids":'downtimeEventIds',
        "status_ids":'statusIds',
        "comments":'comments',
        "recurrence":'recurrence',
        "main_task_id":'mainTaskId',
        "assignee":'assignee',
        "metadata":'metadata'
    }

    def __init__(self,
                 task_id=None,
                 title=None,
                 created_by=None,
                 priority='medium',
                 timestamp_start=None,
                 timestamp_end=None,
                 state=None,
                 resolved=None,
                 number_of_comments=None,
                 description=None,
                 category=None,
                 timestamp_resolved=None,
                 device_ids=None,
                 site_ids=None,
                 site_access_ids=None,
                 downtime_event_ids=None,
                 status_ids=None,
                 comments=None,
                 recurrence=None,
                 main_task_id=None,
                 assignee=None,
                 metadata=None):
        """Constructor for the Task class"""

        # Initialize members of the class
        self.task_id = task_id
        self.title = title
        self.created_by = created_by
        self.description = description
        self.category = category
        self.priority = priority
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None
        self.state = state
        self.resolved = resolved
        self.timestamp_resolved = APIHelper.RFC3339DateTime(timestamp_resolved) if timestamp_resolved else None
        self.device_ids = device_ids
        self.site_ids = site_ids
        self.site_access_ids = site_access_ids
        self.downtime_event_ids = downtime_event_ids
        self.status_ids = status_ids
        self.number_of_comments = number_of_comments
        self.comments = comments
        self.recurrence = recurrence
        self.main_task_id = main_task_id
        self.assignee = assignee
        self.metadata = metadata


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        task_id = dictionary.get('taskId')
        title = dictionary.get('title')
        created_by = greenbyteapi.models.user.User.from_dictionary(dictionary.get('createdBy')) if dictionary.get('createdBy') else None
        priority = dictionary.get("priority") if dictionary.get("priority") else 'medium'
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None
        state = dictionary.get('state')
        resolved = dictionary.get('resolved')
        number_of_comments = dictionary.get('numberOfComments')
        description = dictionary.get('description')
        category = greenbyteapi.models.task_category.TaskCategory.from_dictionary(dictionary.get('category')) if dictionary.get('category') else None
        timestamp_resolved = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampResolved")).datetime if dictionary.get("timestampResolved") else None
        device_ids = dictionary.get('deviceIds')
        site_ids = dictionary.get('siteIds')
        site_access_ids = dictionary.get('siteAccessIds')
        downtime_event_ids = dictionary.get('downtimeEventIds')
        status_ids = dictionary.get('statusIds')
        comments = None
        if dictionary.get('comments') != None:
            comments = list()
            for structure in dictionary.get('comments'):
                comments.append(greenbyteapi.models.task_comment.TaskComment.from_dictionary(structure))
        recurrence = greenbyteapi.models.recurrence.Recurrence.from_dictionary(dictionary.get('recurrence')) if dictionary.get('recurrence') else None
        main_task_id = dictionary.get('mainTaskId')
        assignee = dictionary.get('assignee')
        metadata = None
        if dictionary.get('metadata') != None:
            metadata = list()
            for structure in dictionary.get('metadata'):
                metadata.append(greenbyteapi.models.metadata_field.MetadataField.from_dictionary(structure))

        # Return an object of this model
        return cls(task_id,
                   title,
                   created_by,
                   priority,
                   timestamp_start,
                   timestamp_end,
                   state,
                   resolved,
                   number_of_comments,
                   description,
                   category,
                   timestamp_resolved,
                   device_ids,
                   site_ids,
                   site_access_ids,
                   downtime_event_ids,
                   status_ids,
                   comments,
                   recurrence,
                   main_task_id,
                   assignee,
                   metadata)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class StatusCategoryEnum(object):

    """Implementation of the 'StatusCategory' enum.

    The category a status belongs to.

    Attributes:
        STOP: Statuses indicating that the device is prevented from producing
            power or is in a critical state.
        WARNING: Statuses indicating that the device is not operating
            normally, but is not in a critical state.
        INFORMATIONAL: Statuses that occur during normal operation.
        COMMUNICATION: Statuses related to communication between the device
            and Greenbyte.
        CURTAILMENT: Statuses generated based on curtailment.

    """

    STOP = 'stop'

    WARNING = 'warning'

    INFORMATIONAL = 'informational'

    COMMUNICATION = 'communication'

    CURTAILMENT = 'curtailment'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper

class SiteAccessPersonnel(object):

    """Implementation of the 'SiteAccessPersonnel' model.

    Site access personnel.

    Attributes:
        personnel_id (int): The id of a person in the personnel list.
        first_name (string): TODO: type description here.
        last_name (string): TODO: type description here.
        company (string): TODO: type description here.
        phone_number (string): TODO: type description here.
        vehicle_registration (string): TODO: type description here.
        comment (string): TODO: type description here.
        timestamp_start (datetime): TODO: type description here.
        timestamp_end (datetime): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "personnel_id":'personnelId',
        "first_name":'firstName',
        "last_name":'lastName',
        "company":'company',
        "phone_number":'phoneNumber',
        "vehicle_registration":'vehicleRegistration',
        "comment":'comment',
        "timestamp_start":'timestampStart',
        "timestamp_end":'timestampEnd'
    }

    def __init__(self,
                 personnel_id=None,
                 first_name=None,
                 last_name=None,
                 company=None,
                 phone_number=None,
                 vehicle_registration=None,
                 comment=None,
                 timestamp_start=None,
                 timestamp_end=None):
        """Constructor for the SiteAccessPersonnel class"""

        # Initialize members of the class
        self.personnel_id = personnel_id
        self.first_name = first_name
        self.last_name = last_name
        self.company = company
        self.phone_number = phone_number
        self.vehicle_registration = vehicle_registration
        self.comment = comment
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        personnel_id = dictionary.get('personnelId')
        first_name = dictionary.get('firstName')
        last_name = dictionary.get('lastName')
        company = dictionary.get('company')
        phone_number = dictionary.get('phoneNumber')
        vehicle_registration = dictionary.get('vehicleRegistration')
        comment = dictionary.get('comment')
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None

        # Return an object of this model
        return cls(personnel_id,
                   first_name,
                   last_name,
                   company,
                   phone_number,
                   vehicle_registration,
                   comment,
                   timestamp_start,
                   timestamp_end)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import greenbyteapi.models.client_configuration
import greenbyteapi.models.time_zone_configuration
import greenbyteapi.models.data_signal_configuration

class ConfigurationItem(object):

    """Implementation of the 'ConfigurationItem' model.

    Your configuration data.

    Attributes:
        client (ClientConfiguration): General configuration data.
        time_zone (TimeZoneConfiguration): The time zone configuration.
        data_signals (DataSignalConfiguration): Your data signal
            configuration. These only apply to wind devices.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "client":'client',
        "time_zone":'timeZone',
        "data_signals":'dataSignals'
    }

    def __init__(self,
                 client=None,
                 time_zone=None,
                 data_signals=None):
        """Constructor for the ConfigurationItem class"""

        # Initialize members of the class
        self.client = client
        self.time_zone = time_zone
        self.data_signals = data_signals


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        client = greenbyteapi.models.client_configuration.ClientConfiguration.from_dictionary(dictionary.get('client')) if dictionary.get('client') else None
        time_zone = greenbyteapi.models.time_zone_configuration.TimeZoneConfiguration.from_dictionary(dictionary.get('timeZone')) if dictionary.get('timeZone') else None
        data_signals = greenbyteapi.models.data_signal_configuration.DataSignalConfiguration.from_dictionary(dictionary.get('dataSignals')) if dictionary.get('dataSignals') else None

        # Return an object of this model
        return cls(client,
                   time_zone,
                   data_signals)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class PowerCurveValue(object):

    """Implementation of the 'PowerCurveValue' model.

    The power at a specific wind speed according to a power curve.

    Attributes:
        wind_speed (float): Wind speed in m/s
        power (float): Power in kW

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "wind_speed":'windSpeed',
        "power":'power'
    }

    def __init__(self,
                 wind_speed=None,
                 power=None):
        """Constructor for the PowerCurveValue class"""

        # Initialize members of the class
        self.wind_speed = wind_speed
        self.power = power


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        wind_speed = dictionary.get('windSpeed')
        power = dictionary.get('power')

        # Return an object of this model
        return cls(wind_speed,
                   power)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import dateutil.parser

class Recurrence(object):

    """Implementation of the 'Recurrence' model.

    Recurrence settings for the task. To calculate when the
    task is recurring, use the `timestampStart` field and
    then add to it multiples of the specified interval; the
    `intervalType` field determines if the task is recurring
    on daily, weekly, monthly, or yearly basis.
    If the task is not recurring, this field is null.
    **Note:** Only the main (first) task in a recurring
    series have recurrence settings. For the other tasks in
    the series, the field `mainTaskId` can be used to find
    it.

    Attributes:
        interval (int): The interval with which the task repeats.
        interval_type (IntervalTypeEnum): The type of with which the task
            repeats.
        date_end (date): When the recurring task series ends (exclusive).  The
            end date is **not** included in the recurring task series: for
            example, to have a task series occur until and including the last
            day of March 2020, set `dateEnd` to "2020-04-01".

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "interval":'interval',
        "interval_type":'intervalType',
        "date_end":'dateEnd'
    }

    def __init__(self,
                 interval=None,
                 interval_type=None,
                 date_end=None):
        """Constructor for the Recurrence class"""

        # Initialize members of the class
        self.interval = interval
        self.interval_type = interval_type
        self.date_end = date_end


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        interval = dictionary.get('interval')
        interval_type = dictionary.get('intervalType')
        date_end = dateutil.parser.parse(dictionary.get('dateEnd')).date() if dictionary.get('dateEnd') else None

        # Return an object of this model
        return cls(interval,
                   interval_type,
                   date_end)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class DeviceModel(object):

    """Implementation of the 'DeviceModel' model.

    General device model information.

    Attributes:
        device_model_id (int): TODO: type description here.
        manufacturer (string): TODO: type description here.
        model (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device_model_id":'deviceModelId',
        "manufacturer":'manufacturer',
        "model":'model'
    }

    def __init__(self,
                 device_model_id=None,
                 manufacturer=None,
                 model=None):
        """Constructor for the DeviceModel class"""

        # Initialize members of the class
        self.device_model_id = device_model_id
        self.manufacturer = manufacturer
        self.model = model


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        device_model_id = dictionary.get('deviceModelId')
        manufacturer = dictionary.get('manufacturer')
        model = dictionary.get('model')

        # Return an object of this model
        return cls(device_model_id,
                   manufacturer,
                   model)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.models.user

class TaskComment(object):

    """Implementation of the 'TaskComment' model.

    A comment added to a task.

    Attributes:
        comment_id (int): The id of a task comment.
        text (string): TODO: type description here.
        timestamp_created (datetime): The timestamp when the comment was
            created (added to the task). The timestamp is in the time zone
            configured in the Greenbyte Platform without UTC offset.
        created_by (User): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "comment_id":'commentId',
        "text":'text',
        "timestamp_created":'timestampCreated',
        "created_by":'createdBy'
    }

    def __init__(self,
                 comment_id=None,
                 text=None,
                 timestamp_created=None,
                 created_by=None):
        """Constructor for the TaskComment class"""

        # Initialize members of the class
        self.comment_id = comment_id
        self.text = text
        self.timestamp_created = APIHelper.RFC3339DateTime(timestamp_created) if timestamp_created else None
        self.created_by = created_by


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        comment_id = dictionary.get('commentId')
        text = dictionary.get('text')
        timestamp_created = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampCreated")).datetime if dictionary.get("timestampCreated") else None
        created_by = greenbyteapi.models.user.User.from_dictionary(dictionary.get('createdBy')) if dictionary.get('createdBy') else None

        # Return an object of this model
        return cls(comment_id,
                   text,
                   timestamp_created,
                   created_by)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class TaskAssigneeTypeEnum(object):

    """Implementation of the 'TaskAssigneeType' enum.

    The type of task assignee.

    Attributes:
        USER: A Greenbyte Platform user assignee.
        PERSONNEL: A personnel assignee.
        MANUFACTURER: The manufacturer as assignee.
        OTHER: Any other assignee.

    """

    USER = 'user'

    PERSONNEL = 'personnel'

    MANUFACTURER = 'manufacturer'

    OTHER = 'other'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper

class DowntimeEvent(object):

    """Implementation of the 'DowntimeEvent' model.

    A downtime event.

    Attributes:
        downtime_event_id (int): The id of a downtime event.
        timestamp_start (datetime): The timestamp when the downtime is/was
            planned to start. The timestamp is in the time zone configured in
            the Greenbyte Platform without UTC offset.
        timestamp_end (datetime): The timestamp when the downtime is/was
            planned to end. The timestamp is in the time zone configured in
            the Greenbyte Platform without UTC offset.
        comment (string): A comment describing the downtime event.
        device_ids (list of int): TODO: type description here.
        site_ids (list of int): TODO: type description here.
        task_ids (list of int): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "downtime_event_id":'downtimeEventId',
        "timestamp_start":'timestampStart',
        "timestamp_end":'timestampEnd',
        "comment":'comment',
        "device_ids":'deviceIds',
        "site_ids":'siteIds',
        "task_ids":'taskIds'
    }

    def __init__(self,
                 downtime_event_id=None,
                 timestamp_start=None,
                 timestamp_end=None,
                 comment=None,
                 device_ids=None,
                 site_ids=None,
                 task_ids=None):
        """Constructor for the DowntimeEvent class"""

        # Initialize members of the class
        self.downtime_event_id = downtime_event_id
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None
        self.comment = comment
        self.device_ids = device_ids
        self.site_ids = site_ids
        self.task_ids = task_ids


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        downtime_event_id = dictionary.get('downtimeEventId')
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None
        comment = dictionary.get('comment')
        device_ids = dictionary.get('deviceIds')
        site_ids = dictionary.get('siteIds')
        task_ids = dictionary.get('taskIds')

        # Return an object of this model
        return cls(downtime_event_id,
                   timestamp_start,
                   timestamp_end,
                   comment,
                   device_ids,
                   site_ids,
                   task_ids)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import greenbyteapi.models.data_signal

class DataItem(object):

    """Implementation of the 'DataItem' model.

    An object containing time-series data for a specific aggregate, data
    signal and interval.

    Attributes:
        aggregate (AggregateModeEnum): How data is aggregated in the asset
            structure.
        aggregate_id (int): The id of this aggregate group: device id, site
            id, or the constant -1 for portfolio. For `siteLevel` aggregation
            a generated unique id is used.
        aggregate_path_names (list of string): For `siteLevel` aggregation
            this contains the title for each level in the hierarchy. For other
            types of aggregation it will be empty.
        device_ids (list of int): The ids of the devices in this aggregate
            group.
        resolution (ResolutionEnum): The resolution of the data. Note: For
            requests with *device* as the specified resolution, the actual
            resolution will be returned e.g. *10minute*.
        calculation (CalculationModeEnum): Which operation to use when
            aggregating data.
        data_signal (DataSignal): A data signal.
        data (dict<object, float>): A dictionary with timestamp as key and the
            data measurement as value. The timestamp is in the time zone
            configured in the Greenbyte Platform without UTC offset. Example:
            `{ ..., "2020-01-01T00:00:00": 584.33, ... }`.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "aggregate":'aggregate',
        "aggregate_id":'aggregateId',
        "device_ids":'deviceIds',
        "resolution":'resolution',
        "calculation":'calculation',
        "data_signal":'dataSignal',
        "data":'data',
        "aggregate_path_names":'aggregatePathNames'
    }

    def __init__(self,
                 aggregate='device',
                 aggregate_id=None,
                 device_ids=None,
                 resolution=None,
                 calculation=None,
                 data_signal=None,
                 data=None,
                 aggregate_path_names=None):
        """Constructor for the DataItem class"""

        # Initialize members of the class
        self.aggregate = aggregate
        self.aggregate_id = aggregate_id
        self.aggregate_path_names = aggregate_path_names
        self.device_ids = device_ids
        self.resolution = resolution
        self.calculation = calculation
        self.data_signal = data_signal
        self.data = data


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        aggregate = dictionary.get("aggregate") if dictionary.get("aggregate") else 'device'
        aggregate_id = dictionary.get('aggregateId')
        device_ids = dictionary.get('deviceIds')
        resolution = dictionary.get('resolution')
        calculation = dictionary.get('calculation')
        data_signal = greenbyteapi.models.data_signal.DataSignal.from_dictionary(dictionary.get('dataSignal')) if dictionary.get('dataSignal') else None
        data = dictionary.get('data')
        aggregate_path_names = dictionary.get('aggregatePathNames')

        # Return an object of this model
        return cls(aggregate,
                   aggregate_id,
                   device_ids,
                   resolution,
                   calculation,
                   data_signal,
                   data,
                   aggregate_path_names)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class CategoryEnum(object):

    """Implementation of the 'Category' enum.

    TODO: type enum description here.

    Attributes:
        PICTURES: TODO: type description here.
        REPORTS: TODO: type description here.
        OTHER: TODO: type description here.

    """

    PICTURES = 'Pictures'

    REPORTS = 'Reports'

    OTHER = 'Other'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
from greenbyteapi.configuration import Configuration
from greenbyteapi.controllers.base_controller import BaseController
from greenbyteapi.http.auth.custom_header_auth import CustomHeaderAuth
from greenbyteapi.models.data_signal_item import DataSignalItem
from greenbyteapi.models.data_item import DataItem
from greenbyteapi.models.data_real_time_item import DataRealTimeItem
from greenbyteapi.models.data_per_category_response import DataPerCategoryResponse
from greenbyteapi.exceptions.problem_details_exception import ProblemDetailsException
from greenbyteapi.exceptions.api_exception import APIException

class DataController(BaseController):

    """A Controller to access Endpoints in the greenbyteapi API."""


    def get_data_signals(self,
                         device_ids):
        """Does a GET request to /datasignals.

        Gets authorized data signals for one or more devices.
        _🔐 This endpoint requires the **Data** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `datasignals.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): What devices to get data signals for.

        Returns:
            list of DataSignalItem: Response from the API. The data signals
                available for one or several of the devices.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/datasignals'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DataSignalItem.from_dictionary)

    def get_data(self,
                 device_ids,
                 data_signal_ids,
                 timestamp_start,
                 timestamp_end,
                 use_utc=False,
                 resolution='10minute',
                 aggregate='device',
                 aggregate_level=0,
                 calculation=None):
        """Does a GET request to /data.

        Gets data for multiple devices and data signals in the given
        resolution. The timestamps are in the time zone configured in the
        Greenbyte Platform.
        Use the useUtc flag to get timestamps in UTC for all resolutions other
        than daily, weekly, monthly and yearly.
        _🔐 This endpoint requires the **Data** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `data.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): Which devices to get data for.
            data_signal_ids (list of int): Which data signals to get data
                for.
            timestamp_start (datetime): The beginning of the time interval to
                get data for (inclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                start timestamp **is** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            timestamp_end (datetime): The end of the time interval to get data
                for (exclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                end timestamp is **not** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            use_utc (bool, optional): Set to true to get timestamps in UTC.
                UTC timestamps are available for all resolutions other than
                daily, weekly, monthly and yearly.
            resolution (ResolutionEnum, optional): The desired data
                resolution.
            aggregate (AggregateModeEnum, optional): How the data should be
                aggregated with regards to device(s) or site(s).
            aggregate_level (int, optional): When AggregateMode `siteLevel` is
                used this parameter controls down to which level in the
                hierarchy to aggregate.
            calculation (CalculationModeEnum, optional): The calculation used
                when aggregating data, both over time and across devices. The
                default is the data signal default.

        Returns:
            list of DataItem: Response from the API. The data grouped by data
                signal and aggregate.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/data'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids,
            'dataSignalIds': data_signal_ids,
            'timestampStart': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_start),
            'timestampEnd': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_end),
            'useUtc': use_utc,
            'resolution': resolution,
            'aggregate': aggregate,
            'aggregateLevel': aggregate_level,
            'calculation': calculation
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DataItem.from_dictionary)

    def get_real_time_data(self,
                           device_ids,
                           data_signal_ids,
                           aggregate='device',
                           aggregate_level=0,
                           calculation=None):
        """Does a GET request to /realtimedata.

        Gets the most recent data point for each
        specified device and data signal. The timestamps are in UTC.
        _🔐 This endpoint requires the **Data** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `realtimedata.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): Which devices to get data for.
            data_signal_ids (list of int): Which data signals to get data
                for.
            aggregate (AggregateModeEnum, optional): How the data should be
                aggregated with regards to device(s) or site(s).
            aggregate_level (int, optional): When AggregateMode `siteLevel` is
                used this parameter controls down to which level in the
                hierarchy to aggregate.
            calculation (CalculationModeRealTimeEnum, optional): The
                calculation used when aggregating data, both over time and
                across devices. The default is the data signal default.

        Returns:
            list of DataRealTimeItem: Response from the API. The most recent
                data points grouped by data signal and aggregate.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/realtimedata'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids,
            'dataSignalIds': data_signal_ids,
            'aggregate': aggregate,
            'aggregateLevel': aggregate_level,
            'calculation': calculation
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DataRealTimeItem.from_dictionary)

    def get_data_per_category(self,
                              device_ids,
                              data_signal_id,
                              timestamp_start,
                              timestamp_end,
                              aggregate='device',
                              aggregate_level=0,
                              category=None):
        """Does a GET request to /datapercategory.

        Gets signal data aggregated per availability contract category.
        _🔐 This endpoint requires the **Data** and **Statuses** endpoint
        permissions._
        _This request can also be made using the POST method, 
        with a request to `datapercategory.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): Which devices to get data for.
            data_signal_id (int): Which signal to get data for; only Lost
                Production signals are supported at the moment.
            timestamp_start (datetime): The beginning of the time interval to
                get data for (inclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                start timestamp **is** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            timestamp_end (datetime): The end of the time interval to get data
                for (exclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                end timestamp is **not** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            aggregate (AggregateModeEnum, optional): How the data should be
                aggregated with regards to device(s) or site(s).
            aggregate_level (int, optional): When AggregateMode `siteLevel` is
                used this parameter controls down to which level in the
                hierarchy to aggregate.
            category (list of StatusCategoryEnum, optional): Which status
                categories to include. By default all categories are
                included.

        Returns:
            DataPerCategoryResponse: Response from the API. The data grouped
                by aggregate (device, site, etc.) and contract category.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/datapercategory'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids,
            'dataSignalId': data_signal_id,
            'timestampStart': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_start),
            'timestampEnd': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_end),
            'aggregate': aggregate,
            'aggregateLevel': aggregate_level,
            'category': category
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DataPerCategoryResponse.from_dictionary)



__all__ = [
    'user',
    'data_signal',
    'data_signal_item',
    'data_item',
    'data_real_time_item',
    'data_per_category_response',
    'data_per_category_item',
    'status_item',
    'configuration_item',
    'client_configuration',
    'time_zone_configuration',
    'data_signal_configuration',
    'site',
    'metadata_field',
    'device',
    'device_model',
    'turbine_type',
    'power_curve',
    'power_curve_value',
    'site_with_data',
    'alert_item',
    'device_access',
    'site_access',
    'downtime_event',
    'organization',
    'personnel',
    'personnel_qualification',
    'personnel_site_induction',
    'task',
    'task_category',
    'task_comment',
    'assignee',
    'device_type',
    'recurrence',
    'site_access_personnel',
    'tasks_files_response',
    'resolution_enum',
    'aggregate_mode_enum',
    'calculation_mode_enum',
    'calculation_mode_real_time_enum',
    'status_category_enum',
    'task_assignee_type_enum',
    'task_priority_enum',
    'task_state_enum',
    'category_enum',
    'category_time_enum',
    'interval_type_enum',
]


# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper


class Configuration(object):

    """A class used for configuring the SDK by a user.

    This class need not be instantiated and all properties and methods
    are accessible without instance creation.

    """

    # Set the array parameter serialization method
    # (allowed: indexed, unindexed, plain, csv, tsv, psv)
    array_serialization = "csv"

    # An enum for SDK environments
    class Environment(object):
        # The Greenbyte API for a specific customer
        PRODUCTION = 0

    # An enum for API servers
    class Server(object):
        DEFAULT = 0

    # The environment in which the SDK is running
    environment = Environment.PRODUCTION

    # TODO: Set an appropriate value
    customer = 'intro'

    # TODO: Set an appropriate value
    x_api_key = None


    # All the environments the SDK can run in
    environments = {
        Environment.PRODUCTION: {
            Server.DEFAULT: 'https://{customer}.greenbyte.cloud/api/2',
        },
    }

    @classmethod
    def get_base_uri(cls, server=Server.DEFAULT):
        """Generates the appropriate base URI for the environment and the server.

        Args:
            server (Configuration.Server): The server enum for which the base URI is required.

        Returns:
            String: The base URI.

        """
        parameters = {
            "customer": cls.customer,
        }
        return APIHelper.append_url_with_template_parameters(
            cls.environments[cls.environment][server], parameters, False)



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class DataSignalConfiguration(object):

    """Implementation of the 'DataSignalConfiguration' model.

    Your data signal configuration. These only apply to wind devices.

    Attributes:
        availability_time_data_signal_id (int): The id of the data signal used
            for time-based availability data.
        availability_production_data_signal_id (int): The id of the data
            signal used for production-based availability data.
        lost_production_data_signal_id (int): The id of the data signal used
            for lost production data.
        performance_data_signal_id (int): The id of the data signal used for
            performance data.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "availability_time_data_signal_id":'availabilityTimeDataSignalId',
        "availability_production_data_signal_id":'availabilityProductionDataSignalId',
        "lost_production_data_signal_id":'lostProductionDataSignalId',
        "performance_data_signal_id":'performanceDataSignalId'
    }

    def __init__(self,
                 availability_time_data_signal_id=None,
                 availability_production_data_signal_id=None,
                 lost_production_data_signal_id=None,
                 performance_data_signal_id=None):
        """Constructor for the DataSignalConfiguration class"""

        # Initialize members of the class
        self.availability_time_data_signal_id = availability_time_data_signal_id
        self.availability_production_data_signal_id = availability_production_data_signal_id
        self.lost_production_data_signal_id = lost_production_data_signal_id
        self.performance_data_signal_id = performance_data_signal_id


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        availability_time_data_signal_id = dictionary.get('availabilityTimeDataSignalId')
        availability_production_data_signal_id = dictionary.get('availabilityProductionDataSignalId')
        lost_production_data_signal_id = dictionary.get('lostProductionDataSignalId')
        performance_data_signal_id = dictionary.get('performanceDataSignalId')

        # Return an object of this model
        return cls(availability_time_data_signal_id,
                   availability_production_data_signal_id,
                   lost_production_data_signal_id,
                   performance_data_signal_id)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class CategoryTimeEnum(object):

    """Implementation of the 'CategoryTime' enum.

    TODO: type enum description here.

    Attributes:
        AVAILABLE: TODO: type description here.
        UNAVAILABLE: TODO: type description here.
        EXCLUDED: TODO: type description here.

    """

    AVAILABLE = 'available'

    UNAVAILABLE = 'unavailable'

    EXCLUDED = 'excluded'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from greenbyteapi.api_helper import APIHelper


class AssetsControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(AssetsControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.assets

    # Gets a list of devices that the API key has permissions for.
    #
    #_🔐 This endpoint requires the **Assets** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `devices.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_devices(self):
        # Parameters for the API call
        device_type_ids = APIHelper.json_deserialize('[1,2,3]')
        site_ids = APIHelper.json_deserialize('[1,2,3]')
        parent_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = None
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_devices(device_type_ids, site_ids, parent_ids, fields, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"deviceId":1121,"title":"Enerburg group A","altTitle":null,"identity":nu'
            'll,"site":{"siteId":7,"title":"Enerburg"},"deviceType":"Group","deviceTypeI'
            'd":10,"parentId":null,"childIds":[69,70],"deviceModel":null,"deviceModelId"'
            ':41,"manufacturer":"Generic group","model":"Generic","maxPower":0,"timestam'
            'pStart":"2013-10-01T02:00:00","latitude":"32.36431","longitude":"-88.7037",'
            '"elevation":"0","metadata":[]},{"deviceId":69,"title":"Enerburg 1","altTitl'
            'e":null,"identity":null,"site":{"siteId":7,"title":"Enerburg"},"deviceType"'
            ':"Turbine","deviceTypeId":1,"parentId":1121,"childIds":[],"deviceModel":nul'
            'l,"turbineType":{"turbineTypeId":1,"title":"Enercon E-82 E2 2.3MW","manufac'
            'turer":"Enercon","model":"E-82","controller":"CS82a","ratedPower":2300,"max'
            'RotorSpeed":18},"maxPower":2300,"biddingArea":null,"timestampStart":"2013-1'
            '0-01T02:00:00","latitude":"32.36431","longitude":"-88.7037","elevation":"0"'
            ',"targetAvailability":97,"metadata":[{"key":"Hub Height","value":"120"},{"k'
            'ey":"Direct Drive","value":"no"},{"key":"Blade Heating","value":"yes"}]},{"'
            'deviceId":70,"title":"Enerburg 2","altTitle":null,"identity":null,"site":{"'
            'siteId":7,"title":"Enerburg"},"deviceType":"Turbine","deviceTypeId":1,"pare'
            'ntId":1121,"childIds":[],"deviceModel":null,"turbineType":{"turbineTypeId":'
            '1,"title":"Enercon E-82 E2 2.3MW","manufacturer":"Enercon","model":"E-82","'
            'controller":"CS82a","ratedPower":2300,"maxRotorSpeed":18},"maxPower":2300,"'
            'biddingArea":null,"timestampStart":"2013-10-01T02:00:00","latitude":"32.360'
            '2","longitude":"-88.7194","elevation":"0","targetAvailability":null,"metada'
            'ta":[{"key":"Hub Height","value":"120"},{"key":"Direct Drive","value":"no"}'
            ',{"key":"Blade Heating","value":"yes"}]},{"deviceId":71,"title":"Enerburg 3'
            '","altTitle":null,"identity":null,"site":{"siteId":7,"title":"Enerburg"},"d'
            'eviceType":"Turbine","deviceModel":null,"parentId":null,"childIds":[],"devi'
            'ceTypeId":1,"turbineType":{"turbineTypeId":1,"title":"Enercon E-82 E2 2.3MW'
            '","manufacturer":"Enercon","model":"E-82","controller":"CS82a","ratedPower"'
            ':2300,"maxRotorSpeed":18},"maxPower":2300,"biddingArea":null,"timestampStar'
            't":"2013-10-01T02:00:00","latitude":"32.35697","longitude":"-88.7182","elev'
            'ation":"0","targetAvailability":null,"metadata":[]}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets a list of devices that the API key has permissions for.
    #
    #_🔐 This endpoint requires the **Assets** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `devices.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_devices_1(self):
        # Parameters for the API call
        device_type_ids = APIHelper.json_deserialize('[1,2,3]')
        site_ids = APIHelper.json_deserialize('[1,2,3]')
        parent_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = None
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_devices(device_type_ids, site_ids, parent_ids, fields, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets a list of sites that the API key has permissions
    #for.
    #
    #_🔐 This endpoint requires the **Assets** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `sites.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_sites(self):
        # Parameters for the API call
        fields = APIHelper.json_deserialize('["siteId","title"]')
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.get_sites(fields, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"siteId":1,"title":"Wind farm 1","country":"Sweden","identity":"SE-WF1",'
            '"metadata":[{"key":"Address","value":"Wind Street 123"},{"key":"Phone","val'
            'ue":"555 123 456"}]},{"siteId":2,"title":"Solar site 1","country":"Spain","'
            'identity":"ES-SS1","metadata":[{"key":"Address","value":"Sun Street 456"},{'
            '"key":"Phone","value":"555 456 789"}]}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets a list of sites that the API key has permissions
    #for.
    #
    #_🔐 This endpoint requires the **Assets** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `sites.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_sites_1(self):
        # Parameters for the API call
        fields = APIHelper.json_deserialize('["siteId","title"]')
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.get_sites(fields, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets the default or learned power curves for wind turbines.
    #Other device types are not supported.
    #
    #_🔐 This endpoint requires the **PowerCurves** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `powercurves.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_power_curves(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp = dateutil.parser.parse('2020-01-01').date()
        learned = False

        # Perform the API call through the SDK function
        result = self.controller.get_power_curves(device_ids, timestamp, learned)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"deviceId":1,"title":"Enercon E-82 Noise mode 0","values":[{"windSpeed":'
            '1,"power":0},{"windSpeed":4,"power":88},{"windSpeed":5,"power":205},{"windS'
            'peed":6,"power":371},{"windSpeed":7,"power":601},{"windSpeed":8,"power":901'
            '},{"windSpeed":9,"power":1243},{"windSpeed":10,"power":1591},{"windSpeed":1'
            '1,"power":1876},{"windSpeed":12,"power":1979},{"windSpeed":13,"power":1999}'
            ',{"windSpeed":14,"power":2000},{"windSpeed":15,"power":2000},{"windSpeed":1'
            '6,"power":2000},{"windSpeed":17,"power":2000},{"windSpeed":18,"power":2000}'
            ',{"windSpeed":19,"power":2000},{"windSpeed":20,"power":2000},{"windSpeed":2'
            '1,"power":2000},{"windSpeed":22,"power":2000},{"windSpeed":23,"power":2000}'
            ',{"windSpeed":24,"power":2000},{"windSpeed":25,"power":2000}]}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets the default or learned power curves for wind turbines.
    #Other device types are not supported.
    #
    #_🔐 This endpoint requires the **PowerCurves** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `powercurves.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_power_curves_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp = dateutil.parser.parse('2020-01-01').date()
        learned = False

        # Perform the API call through the SDK function
        result = self.controller.get_power_curves(device_ids, timestamp, learned)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class CalculationModeEnum(object):

    """Implementation of the 'CalculationMode' enum.

    Which operation to use when aggregating data.

    Attributes:
        AVERAGE: Returns the average of all data points in the range.
        SUM: Returns the sum of all data points in the range.
        COUNTER: Returns the raw accumulated counter data.

    """

    AVERAGE = 'average'

    SUM = 'sum'

    COUNTER = 'counter'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.models.site_access_personnel

class SiteAccess(object):

    """Implementation of the 'SiteAccess' model.

    A site access.

    Attributes:
        site_access_id (int): The id of a site access.
        site_id (int): The id of a site.
        device_ids (list of int): Device ids associated with the site access.
        task_ids (list of int): Task ids associated with the site access.
        site_access_personnel (list of SiteAccessPersonnel): Personnel
            associated with the site access.
        timestamp_start (datetime): The timestamp when the site access is/was
            planned to start. The timestamp is in the time zone configured in
            the Greenbyte Platform without UTC offset.
        timestamp_end_expected (datetime): The timestamp when the site access
            is/was planned to end. The timestamp is in the time zone
            configured in the Greenbyte Platform without UTC offset.
        timestamp_end (datetime): The timestamp when the site access actually
            ended. The timestamp is in the time zone configured in the
            Greenbyte Platform without UTC offset.
        log_on_comment (string): A comment for when logging on to the site.
        log_off_comment (string): A comment for when logging off from the
            site.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "site_access_id":'siteAccessId',
        "site_id":'siteId',
        "device_ids":'deviceIds',
        "task_ids":'taskIds',
        "site_access_personnel":'siteAccessPersonnel',
        "timestamp_start":'timestampStart',
        "timestamp_end_expected":'timestampEndExpected',
        "timestamp_end":'timestampEnd',
        "log_on_comment":'logOnComment',
        "log_off_comment":'logOffComment'
    }

    def __init__(self,
                 site_access_id=None,
                 site_id=None,
                 device_ids=None,
                 task_ids=None,
                 site_access_personnel=None,
                 timestamp_start=None,
                 timestamp_end_expected=None,
                 timestamp_end=None,
                 log_on_comment=None,
                 log_off_comment=None):
        """Constructor for the SiteAccess class"""

        # Initialize members of the class
        self.site_access_id = site_access_id
        self.site_id = site_id
        self.device_ids = device_ids
        self.task_ids = task_ids
        self.site_access_personnel = site_access_personnel
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end_expected = APIHelper.RFC3339DateTime(timestamp_end_expected) if timestamp_end_expected else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None
        self.log_on_comment = log_on_comment
        self.log_off_comment = log_off_comment


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        site_access_id = dictionary.get('siteAccessId')
        site_id = dictionary.get('siteId')
        device_ids = dictionary.get('deviceIds')
        task_ids = dictionary.get('taskIds')
        site_access_personnel = None
        if dictionary.get('siteAccessPersonnel') != None:
            site_access_personnel = list()
            for structure in dictionary.get('siteAccessPersonnel'):
                site_access_personnel.append(greenbyteapi.models.site_access_personnel.SiteAccessPersonnel.from_dictionary(structure))
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end_expected = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEndExpected")).datetime if dictionary.get("timestampEndExpected") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None
        log_on_comment = dictionary.get('logOnComment')
        log_off_comment = dictionary.get('logOffComment')

        # Return an object of this model
        return cls(site_access_id,
                   site_id,
                   device_ids,
                   task_ids,
                   site_access_personnel,
                   timestamp_start,
                   timestamp_end_expected,
                   timestamp_end,
                   log_on_comment,
                   log_off_comment)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.configuration import Configuration


class CustomHeaderAuth:

    @staticmethod
    def apply(http_request):
        """ Add custom authentication to the request.

        Args:
            http_request (HttpRequest): The HttpRequest object to which 
                authentication will be added.

        """                
        http_request.add_header("X-Api-Key", Configuration.x_api_key)



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class Organization(object):

    """Implementation of the 'Organization' model.

    An organization used for tasks and personnel.

    Attributes:
        organization_id (int): An id of an organization used for tasks and
            personnel.
        name (string): TODO: type description here.
        email (string): TODO: type description here.
        phone (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "organization_id":'organizationId',
        "name":'name',
        "email":'email',
        "phone":'phone'
    }

    def __init__(self,
                 organization_id=None,
                 name=None,
                 email=None,
                 phone=None):
        """Constructor for the Organization class"""

        # Initialize members of the class
        self.organization_id = organization_id
        self.name = name
        self.email = email
        self.phone = phone


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        organization_id = dictionary.get('organizationId')
        name = dictionary.get('name')
        email = dictionary.get('email')
        phone = dictionary.get('phone')

        # Return an object of this model
        return cls(organization_id,
                   name,
                   email,
                   phone)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from greenbyteapi.api_helper import APIHelper


class AlertsControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(AlertsControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.alerts

    # Gets active alerts for multiple devices.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Alerts** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `activealerts.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_active_alerts(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_active_alerts(device_ids, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"deviceId":179,"ruleId":104,"description":"Power curve less than 90%","d'
            'etails":"Evaluation period: 6 hours\\nData coverage: 70% of devices\\nExclu'
            'de data during: Active Stop or Warning status\\nData condition: The consecu'
            'tive value of the performance index of Backen 2 is less than 90%\\nData con'
            'dition: The consecutive value of the wind speed of Backen 2 is greater than'
            ' 5 m/s","timestampStart":"2020-03-18T06:50:00","timestampEnd":"2020-03-18T1'
            '4:00:00","message":"Low Performance Wind","comment":"A comment"},{"deviceId'
            '":183,"ruleId":104,"description":"Power curve less than 90%","details":"Eva'
            'luation period: 6 hours\\nData coverage: 70% of devices\\nExclude data duri'
            'ng: Active Stop or Warning status\\nData condition: The consecutive value o'
            'f the performance index of Backen 6 is less than 90%\\nData condition: The '
            'consecutive value of the wind speed of Backen 6 is greater than 5 m/s","tim'
            'estampStart":"2020-03-18T07:40:00","timestampEnd":"2020-03-18T14:00:00","me'
            'ssage":"Low Performance Wind","comment":"A comment"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets active alerts for multiple devices.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Alerts** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `activealerts.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_active_alerts_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_active_alerts(device_ids, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets alerts for multiple devices and the given time period.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Alerts** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `alerts.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_alerts(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_alerts(device_ids, timestamp_start, timestamp_end, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"deviceId":179,"ruleId":104,"description":"Power curve less than 90%","d'
            'etails":"Evaluation period: 6 hours\\nData coverage: 70% of devices\\nExclu'
            'de data during: Active Stop or Warning status\\nData condition: The consecu'
            'tive value of the performance index of Backen 2 is less than 90%\\nData con'
            'dition: The consecutive value of the wind speed of Backen 2 is greater than'
            ' 5 m/s","timestampStart":"2020-03-18T06:50:00","timestampEnd":"2020-03-18T1'
            '4:00:00","message":"Low Performance Wind","comment":"A comment"},{"deviceId'
            '":183,"ruleId":104,"description":"Power curve less than 90%","details":"Eva'
            'luation period: 6 hours\\nData coverage: 70% of devices\\nExclude data duri'
            'ng: Active Stop or Warning status\\nData condition: The consecutive value o'
            'f the performance index of Backen 6 is less than 90%\\nData condition: The '
            'consecutive value of the wind speed of Backen 6 is greater than 5 m/s","tim'
            'estampStart":"2020-03-18T07:40:00","timestampEnd":"2020-03-18T14:00:00","me'
            'ssage":"Low Performance Wind","comment":"A comment"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets alerts for multiple devices and the given time period.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Alerts** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `alerts.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_alerts_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_alerts(device_ids, timestamp_start, timestamp_end, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # _**This endpoint is deprecated.** Please use the new endpoint `/activealerts.json` instead._
    def test_test_get_active_alarms(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.get_active_alarms(device_ids, fields, sort_by, sort_asc, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"deviceId":179,"ruleId":104,"description":"Power curve less than 90%","d'
            'etails":"Evaluation period: 6 hours\\nData coverage: 70% of devices\\nExclu'
            'de data during: Active Stop or Warning status\\nData condition: The consecu'
            'tive value of the performance index of Backen 2 is less than 90%\\nData con'
            'dition: The consecutive value of the wind speed of Backen 2 is greater than'
            ' 5 m/s","timestampStart":"2020-03-18T06:50:00","timestampEnd":"2020-03-18T1'
            '4:00:00","message":"Low Performance Wind","comment":"A comment"},{"deviceId'
            '":183,"ruleId":104,"description":"Power curve less than 90%","details":"Eva'
            'luation period: 6 hours\\nData coverage: 70% of devices\\nExclude data duri'
            'ng: Active Stop or Warning status\\nData condition: The consecutive value o'
            'f the performance index of Backen 6 is less than 90%\\nData condition: The '
            'consecutive value of the wind speed of Backen 6 is greater than 5 m/s","tim'
            'estampStart":"2020-03-18T07:40:00","timestampEnd":"2020-03-18T14:00:00","me'
            'ssage":"Low Performance Wind","comment":"A comment"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # _**This endpoint is deprecated.** Please use the new endpoint `/activealerts.json` instead._
    def test_test_get_active_alarms_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.get_active_alarms(device_ids, fields, sort_by, sort_asc, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # _**This endpoint is deprecated.** Please use the new endpoint `/alerts.json` instead._
    def test_test_get_alarms(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.get_alarms(device_ids, timestamp_start, timestamp_end, fields, sort_by, sort_asc, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"deviceId":179,"ruleId":104,"description":"Power curve less than 90%","d'
            'etails":"Evaluation period: 6 hours\\nData coverage: 70% of devices\\nExclu'
            'de data during: Active Stop or Warning status\\nData condition: The consecu'
            'tive value of the performance index of Backen 2 is less than 90%\\nData con'
            'dition: The consecutive value of the wind speed of Backen 2 is greater than'
            ' 5 m/s","timestampStart":"2020-03-18T06:50:00","timestampEnd":"2020-03-18T1'
            '4:00:00","message":"Low Performance Wind","comment":"A comment"},{"deviceId'
            '":183,"ruleId":104,"description":"Power curve less than 90%","details":"Eva'
            'luation period: 6 hours\\nData coverage: 70% of devices\\nExclude data duri'
            'ng: Active Stop or Warning status\\nData condition: The consecutive value o'
            'f the performance index of Backen 6 is less than 90%\\nData condition: The '
            'consecutive value of the wind speed of Backen 6 is greater than 5 m/s","tim'
            'estampStart":"2020-03-18T07:40:00","timestampEnd":"2020-03-18T14:00:00","me'
            'ssage":"Low Performance Wind","comment":"A comment"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # _**This endpoint is deprecated.** Please use the new endpoint `/alerts.json` instead._
    def test_test_get_alarms_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        fields = APIHelper.json_deserialize('["ruleId","timestampStart"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.get_alarms(device_ids, timestamp_start, timestamp_end, fields, sort_by, sort_asc, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

